﻿using AirplaneWithRadar.Generics;
using AirplaneWithRadar.Exceptions;
using Microsoft.Extensions.Logging;
using AirplaneWithRadar.PaintObjects;

namespace AirplaneWithRadar
{
    public partial class FormAirplaneCollection : Form
    {
        private readonly AirplanesGenericStorage storage;

        private readonly ILogger logger;
        public FormAirplaneCollection(ILogger<FormAirplaneCollection> logger)
        {
            InitializeComponent();
            storage = new AirplanesGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            this.logger = logger;
        }

        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(storage.Keys[i].Name);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }
        private void buttonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                logger.LogWarning("Пустое название набора");
                return;
            }
            storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            logger.LogInformation($"Добавлен набор: {textBoxStorageName.Text}");
        }
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowAirplanes();
        }

        private void buttonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                logger.LogWarning("Удаление невыбранного набора");
                return;
            }

            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;

            if (MessageBox.Show($"Удалить объект {name}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                storage.DelSet(listBoxStorages.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
                logger.LogInformation($"Удален набор: {name}");
            }
        }
        private void ButtonAddAirplane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            var formAirplaneConfig = new FormAirplaneConfig();

            formAirplaneConfig.AddEvent(airplane =>
            {
                try
                {
                    if (listBoxStorages.SelectedIndex != -1)
                    {
                        var obj = storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty];
                        if (obj != null)
                        {

                            if (obj + airplane != 1)
                            {
                                MessageBox.Show("Объект добавлен");
                                pictureBoxCollection.Image = obj.ShowAirplanes();
                                logger.LogInformation($"Добавлен объект: {airplane}");
                            }
                            else
                            {
                                MessageBox.Show("Не удалось добавить объект");
                            }
                        }
                    }
                }
                catch (StorageOverflowException ex)
                {
                    MessageBox.Show($"Ошибка при добавлении: {ex.Message}");
                    logger.LogWarning($"Ошибка: {ex.Message}");
                }
                catch (ArgumentException ex)
                {
                    MessageBox.Show($"Ошибка при добавлении: {ex.Message}");
                    logger.LogWarning($"Ошибка: {ex.Message}");
                }
            });
            formAirplaneConfig.Show();
        }

        private void ButtonDeleteAirplane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                logger.LogWarning("Удаление из невыбранного набора");
                return;
            }
            var obj = storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            try
            {
                if (obj - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowAirplanes();
                    logger.LogInformation($"Удалён объект на позиции: {pos}");
                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    logger.LogWarning($"Не удалось удалить объект из набора {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (AirplaneNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                logger.LogWarning($"Ошибка: {ex.Message}");
            }

        }

        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowAirplanes();
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно",
                    "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    logger.LogInformation($"Файл сохранён успешно: {saveFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не сохранилось: {ex.Message}",
                    "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    logger.LogWarning($"Ошибка при сохранении: {ex.Message}");
                }

            }
        }
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка произошла успешно",
                    "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadObjects();
                    logger.LogInformation($"Файл загружен успешно: {openFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не загузилось: {ex.Message}", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    logger.LogWarning($"Ошибка при загрузке: {ex.Message}");
                }

            }

        }

        private void buttonSortByType_Click(object sender, EventArgs e) => CompareAirplanes(new AirplaneCompareByType());


        private void buttonSortByColor_Click(object sender, EventArgs e) => CompareAirplanes(new AirplaneCompareByColor());

        private void CompareAirplanes(IComparer<PaintAirplane?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            obj.SortSet(comparer);
            pictureBoxCollection.Image = obj.ShowAirplanes();
        }
    }
}
